#include <G!/G!.hpp>
#include <Gtl/GrowableArray.hpp>
#include <Screen/ScreenClasses.hpp>
#include <Input/iInput.hpp>
#include <Timer/iTimer.hpp>

class InputReceiver: public GThreadedReceiver
{
	public:
		iScreen		Screen;

		bool 	m_bQuit;
		bool 	m_bViewing;
		bool 	m_bChangeMode;
		bool 	m_bResetMode;
		bool	m_bFullScreen;
		bool 	m_bSetFullScreen;
		UINT32	m_nMode;

		InputReceiver(void);
    	UINT32 ThreadListen(GTransmission &);
        UINT32 WorldTransmission(GTransmission &);
		UINT32 InputTransmission(GTransmission &);
};

bool GATMain(void)
{
    InputReceiver	irClass;
	iInput			Input;
	iTimer			Timer;

	EOEBOOL(Timer);
	EOEBOOL(Input);
	EOEBOOL(irClass.Screen);

    gTransmitter.AddReceiver(irClass);
    Timer->Init(&gTransmitter);
    
    irClass.Screen->SetWindowName("Be3D");
    //irClass.Screen->MakeCurrent();
    //irClass.Screen->SetClearColor(Pixel(200, 200, 200));
	//irClass.Screen->ReleaseCurrent();
	
	while(!irClass.m_bQuit)
	{
		if(irClass.m_bViewing)
		{
			
			//Begin Drawing
			irClass.Screen->MakeCurrent();

			if(!irClass.Screen->Clear())
			{
				Output << "Could not Clear\n";
				Output << irClass.Screen.GetString() << "\n";

				GTransmission transmission(wtsWorld, wtyKill);
    			gTransmitter.Store(transmission);
			}
			irClass.Screen->ReleaseCurrent();
			//End Drawing
			
			if(irClass.m_bSetFullScreen)
			{
				irClass.Screen->GoFullScreen(irClass.m_bFullScreen);
				irClass.m_bSetFullScreen = false;
			}			
	
			if(!irClass.Screen->SwapBuffers())
			{
				Output << "Could not swap buffers\n";
				Output << irClass.Screen.GetString() << "\n";

				GTransmission transmission(wtsWorld, wtyKill);
    			gTransmitter.Store(transmission);
			}
			
			
		}

		if(irClass.m_bChangeMode)
		{
			irClass.m_bChangeMode = false;

			ScreenMode sm = irClass.Screen->AvailableScreenModes()[irClass.m_nMode];
		    irClass.Screen->SetScreenMode(sm);
                       
            if(!irClass.Screen->ChangeMode())
			{
		    	Output << "Screen failed call to init.\n";
				Output << irClass.Screen.GetString() << "\n";
			}
			else
			{
				irClass.Screen->GoFullScreen(irClass.m_bFullScreen);			
				irClass.m_bViewing = true;
			}
		}

		if(irClass.m_bResetMode)
		{

			irClass.m_bResetMode = false;

			if(irClass.m_bFullScreen)
				irClass.Screen->GoFullScreen(false);

		    Output << "Ending ScreenMode.\n";
			if(!irClass.Screen->EndMode())
			{
				Output << "Could not reset mode\n";
				Output << irClass.Screen.GetString() << "\n";
			}
			else
				irClass.m_bViewing = false;
		}

    	gTransmitter.Send();
	}
	
	return true;
}

InputReceiver::InputReceiver(void)
{
	m_bQuit 		= false;
	m_bViewing 		= false;
	m_bChangeMode	= true;
	m_bResetMode	= false;
	m_bFullScreen	= false;
	m_nMode			= 1;
}

UINT32 InputReceiver::ThreadListen(GTransmission &Trans)
{
	switch(Trans.m_nFrom)
	{
		case wtsWorld:
        	return(WorldTransmission(Trans));
		case wtsInput:
        	return(InputTransmission(Trans));
    };

	return(0);
}

UINT32 InputReceiver::WorldTransmission(GTransmission &Trans)
{
	switch(Trans.m_nType)
	{
		case wtyClose:
		case wtyKill:
			m_bQuit = true;
		break;
	};

    return(0);
}

UINT32 InputReceiver::InputTransmission(GTransmission &Trans)
{
	switch(Trans.m_nType)
	{
		case wtyChar:
			//Output << "Received input!!\n";
			GIntAttrib Char;
			if(!Trans.m_dmData.FindAttrib(&Char, Id2Int("CHDN")))
				break;

			if(Char.Int == 'q')
			{
				GTransmission transmission(wtsWorld, wtyKill);
    			gTransmitter.Store(transmission);
			}

			if(Char.Int == 'e')
				m_bResetMode = true;

			if(Char.Int == 'l')
			{
			    Output << "Listing modes.\n";
				for(UINT32 iModes = 0; iModes < Screen->AvailableScreenModes().NumMembers(); iModes++)
				{
					Output << "ScreenMode number:" << iModes << "\n";
					Output << "DeviceID: " << Screen->AvailableScreenModes()[iModes].device_id << "\n";
					Output << "MonitorID: " << Screen->AvailableScreenModes()[iModes].monitor << "\n";
					Output << "Name: " << Screen->AvailableScreenModes()[iModes].name << "\n";
					Output << "Width: " << Screen->AvailableScreenModes()[iModes].pixel_format.width << "\n";
					Output << "Height: " << Screen->AvailableScreenModes()[iModes].pixel_format.height << "\n";
					Output << "Color Depth: "<< Screen->AvailableScreenModes()[iModes].pixel_format.color_depth << "\n";;
					Output << "Z Depth: "<< Screen->AvailableScreenModes()[iModes].pixel_format.z_depth << "\n";;
					Output << "Accum Depth: "<< Screen->AvailableScreenModes()[iModes].pixel_format.accum_depth << "\n";;
					Output << "Stencil Depth: "<< Screen->AvailableScreenModes()[iModes].pixel_format.stencil_depth << "\n";;
					Output << "\n\n";
				}
			}
			
			
			if(Char.Int == 'f')
			{
				m_bFullScreen = true;
				m_bSetFullScreen = true;
			}
			
			if(Char.Int == 'w')
			{
				m_bFullScreen = false;
				m_bSetFullScreen = true;
			}
			
			if(Char.Int == 'v')
				Screen->EnableVsync(true);
			
			
			if(isNum(Char.Int))
			{
			    Output << "Creating screen.\n";
				m_nMode = Char.Int - '0';
				if(m_nMode >= Screen->AvailableScreenModes().NumMembers())
				{
					Output << "Not a supported mode\n";
					break;
				}

				m_bChangeMode = true;
			}

		break;
	};

    return 0;
}